@@ -63,7 +63,7 @@ from optparse import OptionParser
 from trac import scrape, pull_from_trac, TracServer, Config
 from util import (now_str, prune_pending, do_or_die,
                   get_sage_version, current_reports, git_commit,
-                  describe_branch, compare_version, temp_build_suffix,
+                  describe_branch, comparable_version, temp_build_suffix,
                   ensure_free_space,
                   ConfigException, SkipTicket)
 from http_post_file import post_multipart
@@ -302,14 +302,19 @@ def sha1file(path, blocksize=None):
 
     This is used to check spkgs.
 
-    not working in py3
+    should now be working in py3
+
+    EXAMPLES::
+
+        In [2]: sha1file('sage/upstream/tachyon-0.98.9.tar.bz2')
+        Out[2]: '9866dc93e129115994708efa6e7ca16e20d58237'
     """
     if blocksize is None:
         blocksize = 2 ** 16
     h = hashlib.sha1()
-    handle = open(path)
+    handle = open(path, 'rb')
     buf = handle.read(blocksize)
-    while len(buf) > 0:
+    while len(buf):
         h.update(buf)
         buf = handle.read(blocksize)
     handle.close()
@@ -333,8 +338,6 @@ class Patchbot(object):
 
         >>> from patchbot import Patchbot
         >>> P = Patchbot('/homes/leila/sage','http://patchbot.sagemath.org')
-        >>> import os
-        >>> os.chdir(P.sage_root)
         >>> P.test_a_ticket(12345)
 
     How to more or less ban an author: have
@@ -346,6 +349,8 @@ class Patchbot(object):
     def __init__(self, sage_root, server, config_path=None, options=None):
 
         self.sage_root = sage_root
+        os.chdir(self.sage_root)
+
         self.sage_command = os.path.join(self.sage_root, 'sage')
         self.server = server
         self.trac_server = TracServer(Config())
@@ -356,22 +361,16 @@ class Patchbot(object):
         if options is None:
             # this default is for test of patchbot in ipython session
             class opt(object):
-                # two attribute options
                 dry_run = False
-                plugin_only = True
-                # four other options
                 no_banner = False
                 owner = None
+                plugin_only = True
                 safe_only = True
                 skip_base = True
             self.options = opt
         else:
             self.options = options
 
-        # two special options are set as attributes
-        self.dry_run = self.options.dry_run
-        self.plugin_only = self.options.plugin_only
-
         # preparing the logs
         self.log_dir = os.path.join(self.sage_root, 'logs', 'patchbot')
         if not os.path.exists(self.log_dir):
@@ -599,9 +598,11 @@ class Patchbot(object):
                 "max_behind_commits": 0,
                 "max_behind_days": 1.0,
                 "use_ccache": True,
-                # four options that can also be changed using sage --xx
+                # 6 options that can also be changed using sage --xx
+                "dry_run": False,
                 "no_banner": False,
                 "owner": "unknown owner",
+                "plugin_only": False,
                 "safe_only": True,
                 "skip_base": False}
 
@@ -614,11 +615,11 @@ class Patchbot(object):
                          "applies": 20,
                          "behind": 1}
 
-        # now override the default by the config file
+        # now override the default by the json config file
         for key, value in unicode_conf.items():
             conf[str(key)] = value
 
-        # complete back the bonus from config file if needed
+        # complete back the bonus from json config file if needed
         for key, value in default_bonus.items():
             if key not in conf['bonus']:
                 conf['bonus'][key] = value
@@ -631,19 +632,23 @@ class Patchbot(object):
 
         # now override with some values from options
         # coming from sage -patchbot --xx
+        if self.options.dry_run is not None:
+            conf['dry_run'] = self.options.dry_run
         if self.options.no_banner is not None:
             conf['no_banner'] = self.options.no_banner
         if self.options.owner is not None:
             conf['owner'] = self.options.owner
+        if self.options.plugin_only is not None:
+            conf['plugin_only'] = self.options.plugin_only
         if self.options.safe_only is not None:
             conf['safe_only'] = self.options.safe_only
         if self.options.skip_base is not None:
             conf['skip_base'] = self.options.skip_base
 
         # plugin setup
-        if not self.plugin_only:
+        if not conf['plugin_only']:
             active_plugins = conf['plugins']
-            if not "plugins.docbuild" in active_plugins:
+            if "plugins.docbuild" not in active_plugins:
                 # force building the doc, so that the tests can pass
                 conf['plugins'] = active_plugins + ["plugins.docbuild"]
 
@@ -685,9 +690,8 @@ class Patchbot(object):
 
         This will update the patchbot/base if necessary.
         """
-        self.write_log("Check base.", LOG_MAIN)
-        cwd = os.getcwd()
         os.chdir(self.sage_root)
+        self.write_log("Check base.", LOG_MAIN)
         try:
             do_or_die("git checkout patchbot/base")
         except Exception:
@@ -701,17 +705,15 @@ class Patchbot(object):
         only_in_upstream = int(subprocess.check_output(["git", "rev-list", "--count", "patchbot/base..patchbot/base_upstream"]))
 
         max_behind_time = self.config['max_behind_days'] * 60 * 60 * 24
-        if (only_in_base > 0
-                or only_in_upstream > self.config['max_behind_commits']
-                or (only_in_upstream > 0 and
-                    time.time() - self.last_pull < max_behind_time)):
+        if (only_in_base > 0 or
+                only_in_upstream > self.config['max_behind_commits'] or
+                (only_in_upstream > 0 and
+                 time.time() - self.last_pull < max_behind_time)):
             do_or_die("git checkout patchbot/base_upstream")
             do_or_die("git branch -f patchbot/base patchbot/base_upstream")
             do_or_die("git checkout patchbot/base")
             self.last_pull = time.time()
-            os.chdir(cwd)
             return False
-        os.chdir(cwd)
         return True
 
     def human_readable_base(self):
@@ -720,9 +722,10 @@ class Patchbot(object):
         """
         # TODO: Is this stable?
         version = get_sage_version(self.sage_root)
-        commit_count = subprocess.check_output(['git', 'rev-list', '--count',
-                                                '%s..patchbot/base' % version])
-        return "{} + {} commits".format(version, commit_count.strip())
+        commit_count = int(subprocess.check_output(['git', 'rev-list',
+                                                    '--count',
+                                                    '%s..patchbot/base' % version]))
+        return "{} + {} commits".format(version, commit_count)
 
     def get_one_ticket(self, status='open', verbose=0):
         """
@@ -773,6 +776,7 @@ class Patchbot(object):
 
         Return nothing when the ticket should not be tested.
         """
+        os.chdir(self.sage_root)
         log_rat_path = os.path.join(self.log_dir, LOG_RATING)
         with codecs.open(log_rat_path, "a", encoding="utf-8") as log_rating:
 
@@ -819,7 +823,7 @@ class Patchbot(object):
                     self.write_log(msg.format(author),
                                    logfile, False)
                     # self.write_log(msg.format(author).encode('utf-8'),
-                      #              logfile, False) #py2
+                    # logfile, False) #py2
                     return
                 rating += 2 * bonus.get(author, 0)  # bonus for authors
 
@@ -931,7 +935,7 @@ class Patchbot(object):
           * if an integer or a string, use this ticket number
         """
         self.reload_config()
-
+        os.chdir(self.sage_root)
         # ------------- selection of ticket -------------
         if ticket is None:
             rating, ticket = self.get_one_ticket()
@@ -969,7 +973,7 @@ class Patchbot(object):
         print("\n\n")
         log = os.path.join(self.log_dir, '{}-log.txt'.format(ticket['id']))
         self.write_log('#{}: init phase'.format(ticket['id']), [LOG_MAIN, LOG_MAIN_SHORT])
-        if not self.plugin_only:
+        if not self.config['plugin_only']:
             self.report_ticket(ticket, status='Pending', log=log)
         plugins_results = []
         if not self.config['no_banner']:
@@ -987,10 +991,10 @@ class Patchbot(object):
                 # ------------- pull and apply -------------
                 pull_from_trac(self.sage_root, ticket['id'], force=True,
                                use_ccache=self.config['use_ccache'],
-                               safe_only=self.options.safe_only)
+                               safe_only=self.config['safe_only'])
                 t.finish("Apply")
                 state = 'applied'
-                if not self.plugin_only:
+                if not self.config['plugin_only']:
                     self.report_ticket(ticket, status='Pending',
                                        log=log, pending_status=state)
 
@@ -1015,7 +1019,7 @@ class Patchbot(object):
                     # doc is made later in a plugin
                     t.finish("Build")
                     state = 'built'
-                    if not self.plugin_only:
+                    if not self.config['plugin_only']:
                         self.report_ticket(ticket, status='Pending',
                                            log=log, pending_status=state)
 
@@ -1029,15 +1033,15 @@ class Patchbot(object):
                         "patches": [os.path.join(patch_dir, p)
                                     for p in os.listdir(patch_dir)],
                         "sage_binary": self.sage_command,
-                        "dry_run": self.dry_run,
+                        "dry_run": self.config['dry_run'],
                     }
                     # the keyword "patches" is used in plugin commit_messages
 
                     for name, plugin in self.config['plugins']:
                         try:
-                            if ticket['id'] != 0 and os.path.exists(os.path.join(self.log_dir, '0', name)):
-                                baseline = pickle.load(open(os.path.join(self.log_dir, '0', name)))
-
+                            plug0log = os.path.join(self.log_dir, '0', name)
+                            if ticket['id'] != 0 and os.path.exists(plug0log):
+                                baseline = pickle.load(open(plug0log, 'rb'))
                             else:
                                 baseline = None
                             print(boundary(name, 'plugin'))
@@ -1056,7 +1060,7 @@ class Patchbot(object):
                                                               str(ticket['id']))
                                     if not os.path.exists(plugin_dir):
                                         os.mkdir(plugin_dir)
-                                    pickle.dump(res.baseline, open(os.path.join(plugin_dir, name), 'w'))
+                                    pickle.dump(res.baseline, open(os.path.join(plugin_dir, name), 'wb'))
                                     passed = res.status == PluginResult.Passed
                                     print("{} {}".format(name, res.status))
                                     plugins_results.append((name, passed,
@@ -1071,11 +1075,11 @@ class Patchbot(object):
                                        pending_status='plugins_passed'
                                        if plugins_passed else 'plugins_failed')
 
-                    if self.plugin_only:
+                    if self.config['plugin_only']:
                         state = 'plugins' if plugins_passed else 'plugins_failed'
                     else:
                         # ------------- run tests -------------
-                        if self.dry_run:
+                        if self.config['dry_run']:
                             test_target = os.path.join(self.sage_root,
                                                        "src/sage/misc/a*.py")
                         else:
@@ -1126,7 +1130,7 @@ class Patchbot(object):
                                LOG_MAIN)
                 self.report_ticket(ticket, status=status[state], log=log,
                                    plugins=plugins_results,
-                                   dry_run=self.dry_run)
+                                   dry_run=self.config['dry_run'])
                 self.write_log("Done reporting #{}".format(ticket['id']), LOG_MAIN)
                 break
             except IOError:
@@ -1256,41 +1260,53 @@ class Patchbot(object):
                       dry_run=False, pending_status=None):
         """
         Report about a ticket.
+
+        INPUT:
+
+        - ticket -- the ticket id, for example '21345'
+        - status -- can be 'Pending', 'TestsPassed', etc
+        - log -- the log
+        - plugins -- results of the plugins
+        - dry_run -- ?
+        - pending_status -- can be 'applied', 'built', 'plugins_passed',
+          'plugins_failed', etc
         """
-        report = {
-            'status': status,
-            'deps': ticket['depends_on'],
-            'spkgs': ticket['spkgs'],
-            'base': self.base,
-            'user': self.config['user'],
-            'machine': self.config['machine'],
-            'time': now_str(),
-            'plugins': plugins,
-            'patchbot_version': self._version,
-        }
+        report = {'status': status,
+                  'deps': ticket['depends_on'],
+                  'spkgs': ticket['spkgs'],
+                  'base': self.base,
+                  'user': self.config['user'],
+                  'owner': self.config['owner'],
+                  'machine': self.config['machine'],
+                  'time': now_str(),
+                  'plugins': plugins,
+                  'patchbot_version': self._version}
+
         if pending_status:
             report['pending_status'] = pending_status
         try:
             tags = [describe_branch('patchbot/base', tag_only=True),
                     describe_branch('patchbot/ticket_upstream', tag_only=True)]
-            report['base'] = ticket_base = sorted(tags, compare_version)[-1]
+            report['base'] = ticket_base = sorted(tags, key=comparable_version)[-1]
             report['git_base'] = self.git_commit('patchbot/base')
             report['git_base_human'] = describe_branch('patchbot/base')
-            if ticket['id'] == 0:
-                report['owner'] = self.owner
-            if ticket['id'] != 0:
+            if ticket['id'] != 0:  # not on fake ticket 0
                 report['git_branch'] = ticket.get('git_branch', None)
-                report['git_log'] = subprocess.check_output(['git', 'log', '--oneline', '%s..patchbot/ticket_upstream' % ticket_base]).strip().split('\n')
+                report['git_log'] = subprocess.check_output(['git', 'log', '--oneline', '%s..patchbot/ticket_upstream' % ticket_base], universal_newlines=True).strip().split('\n')
                 # If apply failed, we do not want to be stuck in an
                 # infinite loop.
                 report['git_commit'] = self.git_commit('patchbot/ticket_upstream')
                 report['git_commit_human'] = describe_branch('patchbot/ticket_upstream')
                 report['git_merge'] = self.git_commit('patchbot/ticket_merged')
                 report['git_merge_human'] = describe_branch('patchbot/ticket_merged')
-            else:
+            else:  # on fake ticket 0
                 report['git_branch'] = self.config['base_branch']
+                report['git_commit'] = report['git_base']
+                report['git_commit_human'] = report['git_base_human']
+                report['git_merge'] = report['git_base']
+                report['git_merge_human'] = report['git_base_human']
                 report['git_log'] = []
-                report['git_commit'] = report['git_merge'] = report['git_base']
+
         except Exception:
             traceback.print_exc()
 
@@ -1304,12 +1320,17 @@ class Patchbot(object):
         print("{}: {}".format(ticket['id'], status))
         fields = {'report': json.dumps(report)}
         if os.path.exists(log):
-            files = [('log', 'log', bz2.compress(open(log).read()))]
-            # files = [('log', 'log', bz2.compress(open(log).read().encode('utf-8')))]  # py3
+            # py3 : opens the file, get bytes
+            # py2 : opens the file, get str=bytes
+            local_log = open(log, 'rb').read()
+            compressed = bz2.compress(local_log)
+            files = [('log', 'log', compressed)]
         else:
             files = []
         if not dry_run or status == 'Pending':
-            print(post_multipart("%s/report/%s" % (self.server, ticket['id']), fields, files))
+            print(post_multipart("{}/report/{}".format(self.server,
+                                                       ticket['id']),
+                                 fields, files))
 
     def git_commit(self, branch):
         return git_commit(self.sage_root, branch)
@@ -1351,17 +1372,16 @@ def main(args):
                            " '12345,19876'")
 
     # options that are passed to the patchbot via the class "options"
-    # first two attribute options
+    # these 6 options will be stored in conf
     parser.add_option("--dry-run", action="store_true", dest="dry_run",
                       default=False)
+    parser.add_option("--no-banner", action="store_true", dest="no_banner",
+                      help="whether to print the utf8 banner")
+    parser.add_option("--owner", dest="owner",
+                      help="name and email of the human behind the bot")
     parser.add_option("--plugin-only", action="store_true", dest="plugin_only",
                       default=False,
                       help="run the patchbot in plugin-only mode")
-    # then four other options that will be stored in conf
-    parser.add_option("--no-banner", action="store_true", dest="no_banner",
-                      help="wether to print the utf8 banner")
-    parser.add_option("--owner", dest="owner",
-                      help="name and email of the human behind the bot")
     parser.add_option("--safe-only", action="store_true", dest="safe_only",
                       help="whether to run the patchbot in safe-only mode")
     parser.add_option("--skip-base", action="store_true", dest="skip_base",
@@ -1406,7 +1426,7 @@ def main(args):
 
         def good(report):
             return report['machine'] == conf['machine'] and report['status'] == 'TestsPassed'
-        if options.plugin_only or not any(good(report) for report in patchbot.current_reports(0)):
+        if conf['plugin_only'] or not any(good(report) for report in patchbot.current_reports(0)):
             res = patchbot.test_a_ticket(0)
             if res not in ('TestsPassed', 'PluginOnly'):
                 print("\n\n")
