@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # global python imports
 import os
 import sys
@@ -25,7 +26,7 @@ except ImportError:
 # imports from patchbot sources
 from trac import scrape
 from util import (now_str, current_reports, latest_version,
-                  compare_version)
+                  comparable_version)
 from patchbot import filter_on_authors, prune_pending
 
 import db
@@ -33,7 +34,7 @@ from db import tickets
 
 IMAGES_DIR = '/home/patchbot/sage-patchbot/src/images/'
 # oldest version of sage about which we still care
-OLDEST = '7.0'
+OLDEST = comparable_version('7.1')
 
 # machines that are banned from posting their reports
 BLACKLIST = ['hera-OptiPlex-7010']
@@ -62,7 +63,7 @@ def latest_base(betas=True):
     versions = list(tickets.find({'id': 0}).distinct('reports.base'))
     if not betas:
         versions = list(filter(re.compile(r'[0-9.]+$').match, versions))
-    versions.sort(compare_version)
+    versions.sort(key=comparable_version)
     return versions[-1]
 
 app = Flask(__name__)
@@ -86,10 +87,10 @@ def compute_trusted_authors():
             if a:
                 authors[a] += 1
         # code below is temporary and for backward compatibility only
-        for author in ticket.get("authors", []):
-            a = author.strip()
-            if a:
-                authors[a] += 1
+        # for author in ticket.get("authors", []):
+        #     a = author.strip()
+        #     if a:
+        #         authors[a] += 1
     return authors
 
 
@@ -261,14 +262,13 @@ def ticket_list():
     ticket0 = tickets.find_one({'id': 0})
     base_status = get_ticket_status(ticket0, base)
     versions = list(set(report['base'] for report in ticket0['reports']))
-    versions.sort(compare_version)
-    versions = [v for v in versions if compare_version(v, OLDEST) == 1]
+    versions.sort(key=comparable_version)
+    versions = [v for v in versions if comparable_version(v) >= OLDEST]
     versions = [(v, get_ticket_status(ticket0, v)) for v in versions]
 
     return render_template("ticket_list.html", tickets=preprocess(all),
                            summary=summary, base=base, base_status=base_status,
-                           versions=versions, status_order=status_order,
-                           compare_version=compare_version)
+                           versions=versions, status_order=status_order)
 
 
 class MachineStats:
@@ -432,7 +432,7 @@ def render_ticket(ticket):
                 git_log = item.get('git_log')
                 item['git_log_len'] = '?' if git_log is None else len(git_log)
             item['raw_base'] = item['base']
-            if compare_version(item['base'], latest) < 0:
+            if comparable_version(item['base']) <= comparable_version(latest):
                 item['base'] = "<span style='color: red'>%s</span>" % item['base']
             if 'time' in item:
                 item['log'] = log_name(info['id'], item)
@@ -493,6 +493,8 @@ def render_ticket_status(ticket):
     Return the status image for the given ticket.
 
     This now only renders a png image with the base.
+
+    Should be replaced by an svg picture.
     """
     try:
         if 'fast' in request.args:
@@ -520,10 +522,52 @@ def render_ticket_status(ticket):
     return response
 
 
+@app.route("/ticket/<int:ticket>/base.svg")
+def render_ticket_base_svg(ticket):
+    """
+    Return the svg base version image for the given ticket.
+    """
+    try:
+        if 'fast' in request.args:
+            info = tickets.find_one({'id': ticket})
+        else:
+            info = scrape(ticket, db=db)
+    except:
+        info = tickets.find_one({'id': ticket})
+
+    if 'base' in request.args:
+        base = request.args.get('base')
+    else:
+        base = latest_version(info.get('reports', []))
+
+    if base is None:
+        base = ''
+
+    base = base.replace("alpha", u'α').replace("beta", u'β')
+    split_base = base.split('.')
+    if len(split_base) == 2:
+        v_main = base
+        v_sub = ''
+    elif len(split_base) == 3:
+        x, y, z = split_base
+        v_main = x + '.' + y
+        v_sub = z
+    else:
+        v_main = ''
+        v_sub = ''
+    svg = render_template('icon-Version.svg', version_main=v_main,
+                          version_sub=v_sub)
+    response = make_response(svg)
+    response.content_type = 'image/svg+xml'
+    return response
+
+
 @app.route("/ticket/<int:ticket>/status.svg")
 def render_ticket_status_svg(ticket):
     """
     Return the svg status image for the given ticket.
+
+    This displays the current status (TestsPassed, etc) as an svg icon.
     """
     try:
         if 'fast' in request.args:
@@ -730,6 +774,30 @@ status_colors = {'New': 'white',
                  'Spkg': 'purple'}
 
 
+@app.route('/icon-Version.svg')
+def create_base_image_svg():
+    """
+    Create an svg picture displaying a version number.
+
+    EXPERIMENTAL !
+    """
+    base = request.args.get('base', '7.2.beta8')
+    base = base.replace("alpha", u'α').replace("beta", u'β')
+    split_base = base.split('.')
+    if len(split_base) == 2:
+        v_main = base
+        v_sub = ''
+    else:
+        x, y, z = split_base
+        v_main = x + '.' + y
+        v_sub = z
+    svg = render_template('icon-Version.svg', version_main=v_main,
+                          version_sub=v_sub)
+    response = make_response(svg)
+    response.content_type = 'image/svg+xml'
+    return response
+
+
 @app.route("/blob/<status>")
 def status_image(status):
     """
@@ -917,7 +985,9 @@ def robots():
 @app.route("/favicon.ico")
 def favicon():
     """
-    Return the favicon image as a web page (green icon)
+    Return the favicon image as a web page.
+
+    This is currently a 16 x 16 png version of icon-TestsPassed.svg.
 
     See http://patchbot.sagemath.org/favicon.ico for the result.
 
