@@ -4,18 +4,6 @@ import subprocess
 
 from datetime import datetime
 
-# check_output for Python < 2.7
-
-if "check_output" not in subprocess.__dict__:  # duck punch it in!
-    def check_output(args):
-        process = subprocess.Popen(args, stdout=subprocess.PIPE)
-        output, _ = process.communicate()
-        retcode = process.poll()
-        if retcode:
-            raise subprocess.CalledProcessError(retcode, args[0])
-        return output
-    subprocess.check_output = check_output
-
 temp_build_suffix = "-sage-git-temp-"
 
 # DATE_FORMAT = '%Y-%m-%d %H:%M:%S %z'
@@ -29,8 +17,8 @@ def date_parser(date_string):
 
     EXAMPLES::
 
-        date_parser('2015-07-23 09:00:08')
-        ?
+        In [4]: date_parser('2015-07-23 09:00:08')
+        Out[4]: datetime.datetime(2015, 7, 23, 9, 0, 8)
     """
     return datetime.strptime(date_string[:19], DATE_FORMAT)
 
@@ -120,7 +108,9 @@ def current_reports(ticket, base=None, unique=False, newer=False):
                 seen.add(x)
                 return True
     else:
-        first = lambda x: True
+
+        def first(x):
+            return True
 
     reports = list(ticket['reports'])
     reports.sort(key=lambda a: a['time'])
@@ -129,9 +119,9 @@ def current_reports(ticket, base=None, unique=False, newer=False):
         base = latest_version(reports)
 
     def base_ok(report_base):
-        return (not base or base == report_base
-                or (newer and comparable_version(base) <=
-                    comparable_version(report_base)))
+        return (not base or base == report_base or
+                (newer and comparable_version(base) <=
+                 comparable_version(report_base)))
 
     if ticket['id'] == 0:
         return [rep for rep in reports if base_ok(rep['base'])]
@@ -172,10 +162,12 @@ def git_commit(repo, branch):
     """
     ref = "refs/heads/{}".format(branch)
     try:
-        return subprocess.check_output(["git",
-                                        "--git-dir={}/.git".format(repo),
-                                        "show-ref",
-                                        "--verify", ref]).split()[0]
+        res = subprocess.check_output(["git",
+                                       "--git-dir={}/.git".format(repo),
+                                       "show-ref",
+                                       "--verify", ref],
+                                      universal_newlines=True)
+        return res.split()[0]
     except subprocess.CalledProcessError:
         return None
 
@@ -215,27 +207,6 @@ def comparable_version(version):
     return [maybe_int(s) for s in version.split('.')]
 
 
-def compare_version(a, b):
-    """
-    Compare two versions a and b.
-
-    EXAMPLES::
-
-        In [5]: compare_version('6.4.rc0','6.4')
-        Out[5]: -1
-
-        In [6]: compare_version('6.4.rc0','6.4.beta2')
-        Out[6]: 1
-
-        In [7]: compare_version('6.4','6.3')
-        Out[7]: 1
-
-        In [8]: compare_version('6.3','6.3.1')
-        Out[8]: -1
-    """
-    return cmp(comparable_version(a), comparable_version(b))
-
-
 def get_sage_version(sage_root):
     """
     Get the sage version.
@@ -263,7 +234,9 @@ def describe_branch(branch, tag_only=False):
         '6.6.rc1'
     """
     res = subprocess.check_output(['git', 'describe', '--tags',
-                                   '--match', '[0-9].[0-9]*', branch]).strip()
+                                   '--match', '[0-9].[0-9]*', branch],
+                                  universal_newlines=True)
+    res = res.strip()
     if tag_only:
         return res.split('-')[0]
     else:
